#!/usr/bin/env python
#******************************************************************************
#  $Id: csv2c.py 2472 2014-07-30 19:13:04Z hobu $
# 
#  Project:  GDAL
#  Purpose:  Build the epsg_pcs.csv table with the minimum information
#            required to define a PCS (not including GCS defs). 
#  Author:   Frank Warmerdam, warmerdam@pobox.com
#******************************************************************************
#  Copyright (c) 2002, Frank Warmerdam <warmerdam@pobox.com>
# 
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#******************************************************************************
# 
# $Log$
# Revision 1.3  2004/04/27 20:42:57  warmerda
# fixed up C version
#
# Revision 1.2  2004/03/20 07:31:16  warmerda
# improved filename handling
#
# Revision 1.1  2004/03/20 07:10:21  warmerda
# New
#
#

import string
import sys
import csv_tools
import os.path

###############################################################################
def emit_token_line( fd, field_list, counter, varname ):
    fd.write( 'datafile_rows_t %s_row_%d[] = {' % (varname, counter) )
    for item in field_list:
        item = string.replace(item,'\n',' ')
        item = string.replace(item,'"','\\\"')
        fd.write( '"%s",' % item )
    fd.write( 'NULL};\n' )
    
###############################################################################
def convert_csv_table( csv_filename, c_filename ):

    without_ext, ext = os.path.splitext( csv_filename )
    csv_filename = without_ext + '.csv'
    varname = os.path.basename(without_ext)
    
    structname = varname.replace('.','_')
    
    lines = csv_tools.merge_split_lines(open(csv_filename).readlines())

    c_fd = open( c_filename, 'w' )
    c_fd.write( '#include "geo_incode_defs.h"\n' )

    data_lines = 0
    for line in lines:
        emit_token_line( c_fd, csv_tools.SplitCSVLine(line), data_lines,
                         structname )
        data_lines = data_lines + 1

    c_fd.write( 'datafile_rows_t *%s_rows[] = {' % structname )
    for i in range(data_lines):
        if i != 0:
            c_fd.write(' ,')
        c_fd.write( '%s_row_%d' % (structname, i) )
    c_fd.write( ',NULL};\n' )
    c_fd = None

###############################################################################
# main
if len(sys.argv) < 3:
    print 'Usage: csv2c.py filename.csv filename.c'
    sys.exit( 1 )

for arg in sys.argv[1:]:
    convert_csv_table( sys.argv[1], sys.argv[2] )



#!/usr/bin/env python
#******************************************************************************
#  $Id: csv_tools.py 1820 2010-02-27 04:53:55Z warmerdam $
# 
#  Project:  GDAL
#  Purpose:  Build the epsg_pcs.csv table with the minimum information
#            required to define a PCS (not including GCS defs). 
#  Author:   Frank Warmerdam, warmerdam@pobox.com
# 
#******************************************************************************
#  Copyright (c) 2002, Frank Warmerdam <warmerdam@pobox.com>
# 
#  Permission is hereby granted, free of charge, to any person obtaining a
#  copy of this software and associated documentation files (the "Software"),
#  to deal in the Software without restriction, including without limitation
#  the rights to use, copy, modify, merge, publish, distribute, sublicense,
#  and/or sell copies of the Software, and to permit persons to whom the
#  Software is furnished to do so, subject to the following conditions:
# 
#  The above copyright notice and this permission notice shall be included
#  in all copies or substantial portions of the Software.
# 
#  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
#  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
#  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
#  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
#  DEALINGS IN THE SOFTWARE.
#******************************************************************************

import string

def SplitCSVLine( line ):
    tokens = []
    l = len(line)
    cur_token = ''
    in_quotes = 0
    escaped = 0
    for i in range(l):
        c = line[i]
        if c == '"':
            if escaped == 0 and in_quotes and i < l-1 and line[i+1] == '"':
                escaped = 1
            elif escaped == 1:
                cur_token = cur_token + '"'
                escaped = 0
            else:
                in_quotes = not in_quotes
        elif c == ',' and not in_quotes:
            tokens.append( cur_token )
            cur_token = ''
        else:
            cur_token = cur_token + c
            
    tokens.append(cur_token)

    return tokens

# Merge records split by newlines within quotes.
def merge_split_lines( raw_lines ):
    rest_of_lines = []
    cur_line = ''
    for line in raw_lines:
        cur_line = cur_line + line
        quote_count = string.count(cur_line,'"')
        if quote_count % 2 == 0:
            rest_of_lines.append( cur_line )
            cur_line = ''
    if cur_line != '':
        rest_of_lines.append( cur_line )
    return rest_of_lines
    
class CSVTable:

    def __init__( self ):
        self.fields = []
        self.data = {}
        self.multi = 0

    def add_field( self, field_name, default_value = '' ):
        self.fields.append( field_name )
        for key in self.data.keys():
            self.data[key] = self.data[key] + ',' + default_value

    def read_from_csv( self, filename, multi = 0 ):

        self.multi = multi
        
        fd = open( filename )

        # Process the first line as field names.
        tokens = string.split( string.strip(fd.readline()), ',' )
        for field in tokens:
            self.fields.append(string.upper(string.replace( field, '"', '' )))

        # Load the rest of the files, merging records split by newlines
        # within quotes.
        rest_of_lines = merge_split_lines( fd.readlines() )

        # Build lines into an indexed hash table.
        line_counter = -1
        for line in rest_of_lines:
            line_counter += 1
            
            if line[0] == '#':
                continue
            
            if len(line) > 2:
                key, rest = string.split(line,',', 1 )
                key = int(key)
                if multi:
                    if self.data.has_key(key):
                        self.data[key].append(string.strip(line))
                    else:
                        self.data[key] = [string.strip(line)]
                else:
                    self.data[key] = string.strip(line)
            else:
                print 'Problem reading line %d of file %s:%s\n' \
                      % ( line_counter, filename, line)

    def write_to_csv( self, filename ):
        def_line = ''
        for i in range(len(self.fields)):
            if i > 0:
                def_line = def_line + ','
            def_line = def_line + '"' + self.fields[i] + '"'

        fd = open( filename, 'w' )
        fd.write( def_line + '\n' )

        keys = self.data.keys()
        keys.sort()
        for key in keys:
            fd.write( self.data[key] + '\n' )
        fd.close()

    def escape_field( self, in_text ):
        safe_chars = string.ascii_letters + string.digits + '.- '
        
        out_field = ''
        need_quotes = 0
        for c in in_text:
            if c not in safe_chars:
                need_quotes = 1

            if c == '"':
                # add extra quote for quotes.
                out_field = out_field + '"'
                need_quotes = 1
            
            out_field = out_field + c

        if need_quotes:
            out_field = '"' + out_field + '"'

        return out_field

    def line_to_record( self, line ):
        tokens = SplitCSVLine( line )
        if len(tokens) != len(self.fields):
            print 'CSV lines field count does not match'
            print line
            print len(tokens)
            print len(self.fields)
            print tokens

        record = {}
        for i in range(len(self.fields)):
            record[self.fields[i]] = tokens[i]

        return record

    def record_to_line( self, record ):
        line = ''
        for i in range(len(self.fields)):
            if record.has_key(self.fields[i]):
                field_val = self.escape_field( record[self.fields[i]] )
            else:
                field_val = ''
            if i > 0:
                line = line + ','
            line = line + field_val
        return line

    def add_record( self, key, record ):
        self.data[key] = self.record_to_line(record)
        
    def set_record( self, key, record ):
        self.data[key] = self.record_to_line(record)
        
    def get_record( self, key ):
        return self.line_to_record( self.data[key] )

    def get_records( self, key ):
        lines = self.data[key]
        records = []
        for line in lines:
            records.append( self.line_to_record( line ) )

        return records
        
